import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/** The playing board for this arcade game.
 * <p>This game (and therefore this World) is incomplete, but please feel free to
 * improve and adapt this code.
 * <p>This game uses two coordinate systems:
 * <ul><li>The default <code>setLocation()</code> and <code>getX()</code> / <code>getY()</code>
 * methods.</li>
 * <li>The SlidingActor's notion of <code>slideToBoardPosition()</code> and <code>getBoardX()</code> / <code>getBoardY()</code>.</li></ul>
 * <p>All the media in this project was created by jeremy.wood@mac.com. The artwork (and game design) is heavily
 * inspired by Pac-man (copyright by NAMCO), so it should only be used for instructional purposes. The music is 
 * distributed under a <a href="https://creativecommons.org/licenses/by/3.0/us/">Creative
 * Commons Attribution 3 license</a>.
 * <P>A note to instructors: I recommend reviewing this code to get an idea of how gameplay can work, and then
 * gutting most of it except the SlidingActor and the logic for how the enemies move.
 */
public class Board extends World
{
    /** This is a matrix that represents the current board. Each number in this
     * grid corresponds to a different Actor that we need to add to the board in
     * a specific location.
     * <p>
     * TODO: it'd be great to replace this with dynamic levels.
     * 
     */
    int[][] grid = {{1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
                    {1,7,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,9,1},
                    {1,0,1,0,1,0,1,1,1,1,1,1,1,0,1,0,1,0,1,1,1,0,1},
                    {1,0,0,3,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,4,0,0,1},
                    {1,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,1,1,1,1,1,0,1},
                    {1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1},
                    {1,0,1,0,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,0,1,0,1},
                    {1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
                    {1,0,1,0,1,1,1,0,1,1,1,0,1,1,1,0,1,1,1,0,1,1,1},
                    {1,0,1,0,1,0,0,0,0,0,0,2,0,0,1,0,0,0,0,0,0,0,1},
                    {1,0,0,0,1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,0,1,0,1},
                    {1,1,1,0,0,0,1,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,1},
                    {1,0,0,0,1,1,1,1,1,0,1,1,1,1,0,1,1,1,1,0,1,0,1},
                    {1,0,1,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,1,0,1},
                    {1,0,1,0,1,1,1,0,1,1,1,1,0,1,1,0,1,1,1,0,1,0,1},
                    {1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
                    {1,0,1,1,1,0,1,1,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1},
                    {1,0,0,5,0,0,1,0,0,0,0,1,0,0,0,0,1,0,0,6,0,0,1},
                    {1,0,1,0,1,0,1,0,1,1,1,1,1,1,1,0,1,0,1,0,1,0,1},
                    {1,8,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,10,1},
                    {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1} };

    /** This text box is used to display the score. */
    TextBox scoreText = new TextBox(200, 20);
    
    /** This text box is used to display the current level.
     * This feature is unimplemented, so it always stays at "Level: 1"
     */
    TextBox levelText = new TextBox(90, 20);
    
    /** This text box is used to display the number of lives.
     * This feature is unimplemented, so it always stays at "Lives: 3", even
     * when you lose a life.
     */
    TextBox livesText = new TextBox(90, 20);
    
    /** The current running score. */
    int score = 0;
    
    /** The sound to play at the beginning. The originally intention
     * of this sound was to show text that read something like "Ready", "Set", and "Go".
     * <p>This sound was created in GarageBand.
     */
    GreenfootSound introSound = new GreenfootSound("Intro.mp3");
    
    /** The sound to loop as the game plays. 
     * <p>This sound was created in GarageBand.
      */
    GreenfootSound loopSound = new GreenfootSound("Looping.mp3");
                    
    /**
     * Create a new board.
     */
    public Board()
    {    
        super(460, 450, 1, false); 
        
        populateGrid(grid);
        

        addObject(scoreText, 100, 432);
        addObject(levelText, 225, 432);
        addObject(livesText, 438, 432);
        

        scoreText.setText("Score: 0");
        levelText.setText("Level: 1");
        livesText.setText("Lives: 3");
        
        setActOrder(new Class[] { TextBox.class, Wall.class, Hero.class, Enemy.class });
    }
    
    /** This is called when the game starts. This plays the intro sound.
     */
    public void started() {
        introSound.play();
    }
    
    /** Return the number of visible columns.
     * 
     */
    public int getNumberOfColumns() {
        return grid[0].length;
    }
    
    /** Return the number of visible rows.
     * 
     */
    public int getNumberOfRows() {
        return grid.length;
    }
    
    /** This makes sure we transition from the intro sound to the looping sound.
     * 
     */
    public void act() {
        if(!introSound.isPlaying()) {
            if(!loopSound.isPlaying()) {
                loopSound.playLoop();
            }
        }
    }
    
    /** This iterates over every cell in a grid and calls <code>createActor(int)</code> for
       * each value.
       */
    public void populateGrid(int[][] grid) {
        int s = 20;
        for(int y= 0; y<grid.length; y++) {
            for(int x = 0; x<grid[y].length; x++) {
                Actor actor = createActor(grid[y][x]);
                if(actor!=null)
                    addObject(actor, x*s + s/2, y*s + s/2);
            }
        }
    }
    
    /** Create an Actor for each cell type.
     * 
     */
    public Actor createActor(int type) {
        if(type==1) {
            return new Wall();
        } else if(type==2) {
            return new Hero();
        } else if(type==3) {
            return new Enemy("red");
        } else if(type==4) {
            return new Enemy("yellow");
        } else if(type==5) {
            return new Enemy("green");
        } else if(type==6) {
            return new Enemy("blue");
        } else if(type==7) {
            return new Fruit("cherry");
        } else if(type==8) {
            return new Fruit("grape");
        } else if(type==9) {
            return new Fruit("apple");
        } else if(type==10) {
            return new Fruit("orange");
        }
        return null;
    }
    
    /** Return true if a tile is a wall.
     * 
     */
    public boolean isWall(int tileX,int tileY) {
        return grid[tileY][tileX]==1;
    }
    
    /** If a pellet is available at this coordinate: then we remove it
     * and increase the score.
     */
    public boolean consumeFruit(int x,int y) {
        int s = 20;
        List list = getObjectsAt(x*s+s/2, y*s+s/2, Fruit.class);
        if(list.size()==0) return false;
        
        Fruit p = (Fruit)list.get(0);
        removeObject( p );
        list.remove(0);
        setScore(score + 5);
        return true;
    }
    
    /** Change the score, and update the TextBox actor.
     * 
     */
    public void setScore(int newScore) {
        score = newScore;
        scoreText.setText("Score: "+score);
    }
}
